WalkToggleVisual = LibStub("AceAddon-3.0"):NewAddon("WalkToggleVisual", "AceConsole-3.0", "AceHook-3.0")
local core = WalkToggleVisual

local defaults = {
	profile = {
		locked = false,
		pos = {x=500, y=300},
		walking = {
			display = "show",
			color = {r=1, g=0.45, b=0.18, a=1},
			fontSize = 18,
		},
		running = {
			display = "fade",
			color = {r=0.35, g=0.60, b=1, a=1},
			fontSize = 18,
		},
	},
}
local suboptions = { -- options for each walking/running
	display = {
		name = "Display",
		type = "select",
		order = 0,
		values = {
			["hidden"] = "Always Hide",
			["fade"] = "[NYI] Fade out after 5 seconds",
			["show"] = "Always Show",
			["pulse"] = "[NYI] Always Show and Pulse",
		},
	},
	fontSize = {
		name = "Font Size",
		type = "range",
		order = 10,
		min = 6,
		max = 40,
		step = 1,
	},
	color = {
		name = "Color",
		type = "color",
		order = 20,
		hasAlpha = true,
		set = function(info, r, g, b, a) core.db.profile[info[#info-1]].color = {r=r, g=g, b=b, a=a} core:UpdateDisplay() end,
		get = function(info) return core.db.profile[info[#info-1]].color.r, core.db.profile[info[#info-1]].color.g, core.db.profile[info[#info-1]].color.b, core.db.profile[info[#info-1]].color.a end,
	},
}
local options = {
	name = "WalkToggleVisual",
	type = 'group',
	args = {
		toggle = {
			name = "Toggle Anchor",
			type = 'execute',
			order = 0,
			func = "ToggleAnchor",
			handler = core,
		},
		running = {
			name = "Running",
			type = 'group',
			inline = true,
			order = 10,
			args = suboptions,
			set = function(info, value) core.db.profile.running[info[#info]] = value core:UpdateDisplay() end,
			get = function(info) return core.db.profile.running[info[#info]] end,
		},
		walking = {
			name = "Walking",
			type = 'group',
			inline = true,
			order = 20,
			args = suboptions,
			set = function(info, value) core.db.profile.walking[info[#info]] = value end,
			get = function(info) return core.db.profile.walking[info[#info]] end,
		},
	},
}

core.running = true	-- walking/running flag
core.lastToggle = 0	-- time since last swap; use this to see if we can auto-correct

function core:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("WalkToggleVisualDB", defaults, "Default")
	self:RegisterChatCommand("wtv", function() LibStub("AceConfigDialog-3.0"):Open("WalkToggleVisual") end)
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("WalkToggleVisual", options)
	local ACD = LibStub("AceConfigDialog-3.0")
	ACD:AddToBlizOptions("WalkToggleVisual", "WalkToggleVisual")
	
	self:CreateAnchor();
end

function core:OnEnable()
	self:SecureHook("ToggleRun")
	self:SecureHook("MoveAndSteerStart", "PlayerMove")
	self:SecureHook("MoveBackwardStart", "PlayerMove")
	self:SecureHook("MoveForwardStart", "PlayerMove")
	self:SecureHook("StrafeLeftStart", "PlayerMove")
	self:SecureHook("StrafeRightStart", "PlayerMove")
	self:SecureHook("TurnLeftStart", "PlayerMove")
	self:SecureHook("TurnRightStart", "PlayerMove")
	self:SecureHook("TurnOrActionStart", "PlayerMove")
	self:UpdateDisplay()
	
	self:LoadPos();
	if not self.db.profile.locked then
		self.anchorFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.75);
		self.anchorFrame:EnableMouse(true);
	end	
end

function core:OnDisable()
	
end

function core:CreateAnchor()
	if not self.anchorFrame then
		self.anchorFrame = CreateFrame("Frame", nil, UIParent);
		self.anchorFrame:SetWidth(150);
		self.anchorFrame:SetHeight(20);
		self.anchorFrame:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8,
		});
		self.anchorFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.75);
		self.anchorFrame:SetMovable(true);
		self.anchorFrame:RegisterForDrag("LeftButton");
		self.anchorFrame:SetScript("OnDragStart", function() if not self.db.profile.locked then self.anchorFrame:StartMoving() end end);
		self.anchorFrame:SetScript("OnDragStop", function() self.anchorFrame:StopMovingOrSizing() self:SavePos() end);
		if self.db.profile.locked == true then
			self.anchorFrame:SetBackdropColor(0.1, 0.1, 0.1, 0);
			self.anchorFrame:EnableMouse(false);
		else
			self.anchorFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.75);
			self.anchorFrame:EnableMouse(true);
		end
	end
	
	self:CreateString()
end

function core:LoadPos()
	local x = self.db.profile.pos.x;
	local y = self.db.profile.pos.y;
	local s = self.anchorFrame:GetEffectiveScale();
	self.anchorFrame:ClearAllPoints();
	self.anchorFrame:SetParent(UIParent);
	self.anchorFrame:ClearAllPoints();
	self.anchorFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s);
end

function core:SavePos()
	local s = self.anchorFrame:GetEffectiveScale();
	self.db.profile.pos.x = self.anchorFrame:GetLeft()*s;
	self.db.profile.pos.y = self.anchorFrame:GetTop()*s;
end

-- wtvf = nil
function core:CreateString()
	if self.anchorFrame and not self.stringFrame then
		local frame = CreateFrame("Frame")
		frame:SetPoint("CENTER", self.anchorFrame, "CENTER")
		frame:SetWidth(1300)
		frame:SetHeight(1200)
		frame:SetBackdropColor(1, 0, 0, 0.8)
		frame:SetFrameLevel(100)
		
		local str = frame:CreateFontString(nil, "ARTWORK")
		-- str:SetPoint("CENTER")
		str:SetAllPoints()
		local labelFont, size, flags = GameFontHighlightMedium:GetFont()
		str:SetFont(labelFont, size, "OUTLINE")
		str:SetJustifyH("CENTER")
		str:SetText("WalkToggleVisual")
		
		self.stringFrame = frame
		self.stringFrame.main = str
		-- wtvf = frame
	end
end

-- end gui

function core:ToggleAnchor()	
	if self.db.profile.locked then
		self.db.profile.locked = false
		self.anchorFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.75);
		self.anchorFrame:EnableMouse(true);
	else
		self.db.profile.locked = true;
		self.anchorFrame:SetBackdropColor(0.1, 0.1, 0.1, 0);
		self.anchorFrame:EnableMouse(false);
	end
end

function core:ToggleRun()
	self.lastToggle = GetTime()
	
	if HasFullControl() and not UnitDebuff("player", "Dominion") then
		self.running = not self.running
		self:UpdateDisplay()
	end
end

function core:PlayerMove()
	-- only going to auto-correct if we tried to toggle within the last 15 seconds
	-- otherwise, there's a good chance it was a slow applied that is messing with us
	if GetTime()-self.lastToggle > 15 then
		return
	end
	
	local speed = GetUnitSpeed("player")
	if speed < 3 and speed > 0 and self.running then
		self.running = false
		self:UpdateDisplay()
	elseif speed > 3 and not self.running then
		self.running = true
		self:UpdateDisplay()
	end
end

function core:UpdateDisplay()
	local settings = self.running and self.db.profile.running or self.db.profile.walking
	local strobj = self.stringFrame.main
		
	strobj:SetText(self.running and "Running" or "Walking")
	local labelFont, size, flags = GameFontHighlightMedium:GetFont()
	strobj:SetFont(labelFont, settings.fontSize, "OUTLINE")
	strobj:SetTextHeight(settings.fontSize)
	strobj:SetTextColor(settings.color.r, settings.color.g, settings.color.b, settings.color.a)
	
	if settings.display == "hidden" then
		self.stringFrame:Hide()
	else
		self.stringFrame:Show()
		if settings.display == "fade" then
		
		end
	end
end